@@ -1,11 +1,12 @@
 //! Structures represent corresponding fields in ripple transactions.
 
 pub mod account;
-pub mod address;
 pub mod definition;
 pub mod amount;
 pub mod path_set;
 pub mod hash;
 pub mod blob;
-pub mod array;
-pub mod objects;
+pub mod starray;
+pub mod stobject;
+
+
